package web.multitask.trismegistoservices.services.BBVA;

import okhttp3.*;
import org.json.JSONObject;
import org.springframework.stereotype.Service;
import web.multitask.trismegistoservices.utils.CommonUtils;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.util.zip.GZIPInputStream;

@Service
public class bbvaService {

    String globalDate = CommonUtils.getCurrentDateRFC1123();

    public JSONObject getToken(String json) {
        try {
            JSONObject obj = new JSONObject(json);
            obj.remove("body");
            String host = obj.getString("host");
            String formBody = CommonUtils.jsonToUrlEncoded(obj);

            OkHttpClient client = new OkHttpClient();
            Request request = new Request.Builder()
                    .url("https://"+host + "/auth/oauth/v2/token")
                    .post(RequestBody.create(formBody, MediaType.parse("application/x-www-form-urlencoded")))
                    .build();

            Call call = client.newCall(request);
            Response response = call.execute();
            assert response.body() != null;
            String bodyResult = response.body().string();
            response.close();
            return new JSONObject(bodyResult);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return new JSONObject().put("status", false).put("message", e.getMessage());
        }
    }

    public String bodyIntoBase64AfterSHA512(String json) {
        try {
            JSONObject jsonObject = new JSONObject(json);
            String body = jsonObject.optString("body", "");
            byte[] bodyBytes = body.getBytes(StandardCharsets.UTF_8);
            MessageDigest digest = MessageDigest.getInstance("SHA-512");
            byte[] hashedBytes = digest.digest(bodyBytes);
            return Base64.getEncoder().encodeToString(hashedBytes);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    public String txtToSign(String json, String digest) {
        JSONObject obj = new JSONObject(json );
        String host = obj.getString("host");
        String endpoint = obj.getString("endpoint");
        String method = obj.getString("method");

        return "(request-target):" + method.toLowerCase() + " " + endpoint + "\n"
                + "host:" + host + "\n"
                + "date:" + globalDate + "\n"
                + "digest:SHA-512=" + digest;
    }

    public String createSignature(String txt) {
        PrivateKey privateKey = loadPrivateKey("/tokens/key.p8");
        return signContent(txt, privateKey);
    }

    public PrivateKey loadPrivateKey(String path) {
        try(InputStream in = bbvaService.class.getResourceAsStream( path)) {
            String keyString = CommonUtils.readInputStream(in);
            return loadPrivateKeyFromString(keyString);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    private static PrivateKey loadPrivateKeyFromString(String keyString) throws Exception {
        keyString = keyString.replace("-----BEGIN RSA PRIVATE KEY-----", "")
                .replace("-----END RSA PRIVATE KEY-----", "")
                .trim();

        byte[] keyBytes = Base64.getMimeDecoder().decode(keyString);
        PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
        return keyFactory.generatePrivate(spec);
    }

    public String signContent(String content, PrivateKey privateKey) {
        try {
            Signature signature = Signature.getInstance("SHA512withRSA");
            signature.initSign(privateKey);
            signature.update(content.getBytes());
            byte[] signedBytes = signature.sign();
            return Base64.getEncoder().encodeToString(signedBytes);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }


    public JSONObject generateQR(String token,String digest,String signature,String json) {
        try {

            JSONObject obj = new JSONObject(json);
            JSONObject body = obj.getJSONObject("body");
            String host = obj.getString("host");

            MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
            RequestBody requestbody = RequestBody.create(body.toString(), mediaType);
            OkHttpClient client = new OkHttpClient();
            Request request = new Request.Builder()
                    .url("https://" + host + "/pe/request-to-pay/v1/qr")
                    .post(requestbody)
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Accept-Encoding", "gzip")
                    .addHeader("Authorization", "Bearer " + token)
                    .addHeader("Digest", "SHA-512="+digest)
                    .addHeader("Signature", "algorithm=\"rsa-sha512\",headers=\"(request-target) host date digest\",signature=\""+signature+"\"")
                    .addHeader("Host",obj.getString("host"))
                    .addHeader("Date", globalDate)
                    .build();

            Response response = client.newCall(request).execute();
            String bodyResult;
            System.out.println("x-request-id = "+response.header("x-request-id"));
            if (response.body() != null) {
                String encoding = response.header("Content-Encoding", "");
                if ("gzip".equalsIgnoreCase(encoding)) {
                    bodyResult = decompressGzip(response.body().byteStream());
                } else {
                    bodyResult = response.body().string();
                }
            } else {
                bodyResult = "{}";
            }
            System.out.println(bodyResult);
            response.close();
            return new JSONObject(bodyResult);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return new JSONObject().put("status", false).put("message", e.getMessage());
        }
    }

    private String decompressGzip(InputStream compressed) throws IOException {
        try (GZIPInputStream gis = new GZIPInputStream(compressed);
             BufferedReader reader = new BufferedReader(new InputStreamReader(gis, "UTF-8"))) {
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line);
            }
            return sb.toString();
        }
    }
}
